// -*- C++ -*- (c) 2016 Henrich Lauko <xlauko@mail.muni.cz>
#pragma once

DIVINE_RELAX_WARNINGS
#include <llvm/IR/PassManager.h>
DIVINE_UNRELAX_WARNINGS

#include <lart/support/pass.h>
#include <lart/support/meta.h>

#include <lart/abstract/vpa.h>
#include <lart/abstract/decast.h>
#include <lart/abstract/syntactic.h>
#include <lart/abstract/branching.h>
#include <lart/abstract/stash.h>
#include <lart/abstract/assume.h>
#include <lart/abstract/bcp.h>
#include <lart/abstract/concretization.h>
#include <lart/abstract/tainting.h>
#include <lart/abstract/synthesize.h>
#include <lart/abstract/interrupt.h>
#include <lart/abstract/cleanup.h>

namespace lart::abstract {

    using SubstitutionPass = ChainedPass< Concretization, Tainting, Synthesize >;

    struct Abstraction {

        std::string _domain;
        bool _explicit = false;

        Abstraction( std::string opt )
        {
            size_t pos = 0;
            std::string delimiter = ":";

            std::vector< std::string > options;
            while ( (pos = opt.find( delimiter )) != std::string::npos ) {
                options.push_back( opt.substr( 0, pos ) );
                opt.erase( 0, pos + delimiter.length() );
            }

            options.push_back( opt.substr( 0, pos ) );

            assert( options.size() > 0 );
            _domain = options[ 0 ];

            if ( options.size() > 1 && options[ 1 ] == "explicit" ) {
                _explicit = true;
            }

        }

        void run( llvm::Module & m ) {
            auto passes = make_chained_pass( CreateAbstractMetadata( _domain )
                                           , VPA()
                                           , Decast()
                                           , VPA() // run once more to propagate through decasted functions
                                           , StashingPass()
                                           , Syntactic()
                                           , LowerToBool()
                                           , AddAssumes()
                                           , Concretization()
                                           , Tainting()
                                           , Synthesize( _explicit )
                                           , CallInterrupt()
                                           , Cleanup()
                                           );
            passes.run( m );
        }

    };

    static inline PassMeta passes() {
        return passMetaO< Abstraction >( "abstraction", "abstract annotated values" );
    }

} // namespace lart::abstract

